#!/usr/bin/ruby
##############################################################################
## Author: Robert Miesen
## Date Created: 07/30/2014
## Time-stamp:  <07/31/2014 02:26:23 rmiesen>
##
## Comments:
## This program assumes that you have the 'geoip-database' Debian package
## installed, or your platform's equivilent, installed at /usr/share/GeoIP/
## If this assumption is unreasonable, please file a bug report agiainst this
## program with a suggestion on how to fix it for your platform.
##############################################################################

require 'rubygems'
require 'commander/import'
require 'ascii_charts'
require 'awesome_print'
require 'geoip'
require_relative '../lib/fail2ban/geoip/version'



program :version, Fail2ban::Geoip::VERSION
program :description, 'Identify the country of origin (*cough* China *cough*) for an IP address that got banned by fail2ban'
default_command :run

 
command :run do |c|
  c.syntax = 'identifyCountryOfFail2BanBanees.rb run [options]'
  c.summary = ''
  c.description = ''
  c.example 'identifyCountryOfFail2BanBanees.rb run', 'command example'
  c.option  '-l FILE', '--fail2ban-log FILE', String, 'Specify an alternative location for the fail2ban log to be analyzed. Defaults to /var/log/fail2ban.log'

  c.option  '--[no-]ascii-chart-output', 'Output an ascii chart aggregating each countries number of banees. On by default.'

  c.option '--[no-]awesome-print-output', 'Outputs the details of the banee\'s country of origin'
  c.option '--force-color-output', "Forces the use of ascii color output. Useful for piping to other programs that can parse ascii color output, such as less with the '-r' option supplied"
  
  c.action do |args, options|
    options.default(:fail2ban_log => "/var/log/fail2ban.log", :ascii_chart_output => true)
    $fail2ban_log = options.fail2ban_log
    
##LEFT_OFF_HERE: Put under rspec tests, handle cases where the $fail2ban_log file doesn't exist.
    geoip_contry_database = GeoIP.new('/usr/share/GeoIP/GeoIP.dat')
    
    fail2ban_banees = File.readlines($fail2ban_log).collect do |s|
      s =~ /Ban ((?:\d{1,3}\.?){4})/
      $1
    end.delete_if { |s| s == nil }

    countries_and_ips_banned_by_fail2ban = Hash.new
    countries_and_ips_banned_by_fail2ban.default_proc = lambda {|h,k| h[k] = Array.new}
    fail2ban_banees.each do |ip|
      banee_country = geoip_contry_database.country(ip)
      countries_and_ips_banned_by_fail2ban[banee_country[:country_name]] << ip
    end
    
    if options.ascii_chart_output
      puts AsciiCharts::Cartesian.new(countries_and_ips_banned_by_fail2ban.collect {|k,v| [k,v.count]}, :bar => true, :hide_zero => true, :title => "Number of Banees per country").draw
      puts "\n\n"
    end

    if options.awesome_print_output
      AwesomePrint::force_colors! options.force_color_output
      puts "Detailed bannee information:"
      ap countries_and_ips_banned_by_fail2ban, :indent => -2
    end
  end
end

